import logging
from pathlib import Path

from PIL import Image
from roboflow2huggingface.other_utils import add_text_to_image

LOGGER = logging.getLogger(__name__)


def upload_dataset_to_hfhub(dataset_dir, repo_id, token, private=False):
    """
    Uploads a dataset to the Hugging Face Hub.

    Args:
        dataset_dir (str): Path to the dataset directory.
        repo_id (str): The name of the repository to upload to.
        token (str): The token to use to authenticate to the Hugging Face Hub.
        private (bool, optional): Whether the repository should be private. Defaults to False.
    """
    from huggingface_hub import upload_folder, create_repo

    LOGGER.info(f"Uploading dataset to hf.co/{repo_id}...")

    create_repo(
        repo_id=repo_id,
        token=token,
        private=private,
        exist_ok=True,
        repo_type="dataset",
    )
    upload_folder(
        folder_path=dataset_dir,
        repo_id=repo_id,
        token=token,
        repo_type="dataset",
        commit_message="dataset uploaded by roboflow2huggingface package",
    )

    LOGGER.info(f"Dataset uploaded to hf.co/{repo_id}!")


def export_thumbnail(image_path, repo_id, task="object-detection") -> Path:
    """
    Generate thumbnail for the model card

    USERNAME/garbage-object-detection > Garbage Object Detection
    """
    thumbnail_text = repo_id.split("/")[-1]
    texts = thumbnail_text.split("-")
    for ind, text in enumerate(texts):
        if "yolo" not in text.lower():
            texts[ind] = text.title()

    thumbnail_text = " ".join(texts)

    image = add_text_to_image(
        text=thumbnail_text,
        pil_image=Image.open(image_path),
        brightness=0.60,
        text_font=50,
        crop_margin=150,
    )

    if task in ["object-detection", "instance-segmentation"]:
        folder_path = Path(image_path).parent
        thumbnail_path = folder_path / "thumbnail.jpg"
    elif task == "image-classification":
        folder_path = Path(image_path).parent.parent
        thumbnail_path = folder_path / "thumbnail.jpg"

    image.save(str(thumbnail_path), quality=100)

    return thumbnail_path



import os
from pathlib import Path
import yaml


def make_dataset_yolov5_trainable(roboflow_dataset_dir: str) -> str:
    """
    Makes a COCO formatted and zipped Roboflow dataset trainable with YOLOv5.

    Args:
        roboflow_dataset_dir (str): The directory of the Roboflow dataset.

    Returns:
        data.yaml path (str)
    """
    train_zip = Path(roboflow_dataset_dir) / "data" / "train.zip"
    val_zip = Path(roboflow_dataset_dir) / "data" / "valid.zip"

    train_dir = Path(roboflow_dataset_dir) / "data" / "train"
    val_dir = Path(roboflow_dataset_dir) / "data" / "valid"

    train_dir.mkdir(parents=True, exist_ok=True)
    val_dir.mkdir(parents=True, exist_ok=True)

    import zipfile

    with zipfile.ZipFile(train_zip, "r") as zip_ref:
        zip_ref.extractall(train_dir)

    with zipfile.ZipFile(val_zip, "r") as zip_ref:
        zip_ref.extractall(val_dir)

    train_zip.unlink()
    val_zip.unlink()

    # create data.yaml
    datayaml_path = Path(os.path.abspath(roboflow_dataset_dir)) / "data.yaml"
    train_json_path = Path(train_dir) / "_annotations.coco.json"
    val_json_path = Path(val_dir) / "_annotations.coco.json"

    # create yolov5 data yaml (use replace for windows paths)
    data = {
        "train_image_dir": os.path.abspath(train_dir).replace("\\", "/"),
        "val_image_dir": os.path.abspath(val_dir).replace("\\", "/"),
        "train_json_path": os.path.abspath(train_json_path).replace("\\", "/"),
        "val_json_path": os.path.abspath(val_json_path).replace("\\", "/"),
    }
    with open(datayaml_path, "w") as outfile:
        yaml.dump(data, outfile, default_flow_style=False)

    return str(datayaml_path).replace("\\", "/")



from roboflow2huggingface.roboflow_utils import read_roboflow_info


def export_hf_instance_segmentation_dataset_script(
    hf_dataset_id,
    dataset_labels,
    roboflow_dataset,
    export_dir,
    roboflow_universe_url=None,
):
    """
    Exports a HuggingFace dataset script for a Roboflow object detection dataset.

    Args:
        hf_dataset_id (str): HuggingFace dataset id
        dataset_labels (list): List of labels as strings
        roboflow_dataset (roboflow.core.version.Version): Roboflow dataset object
        export_dir (str): Directory to export the dataset script to
    """
    from pathlib import Path

    license, dataset_url, citation, _, _ = read_roboflow_info(
        local_data_dir=export_dir, roboflow_universe_url=roboflow_universe_url
    )
    dataset_name = hf_dataset_id.split("/")[-1]

    urls = (
        f"""{{
                "train": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/train.zip",
                "validation": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/valid.zip",
                "test": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/test.zip",
            }}"""
        if "test" in roboflow_dataset.splits.keys()
        else f"""{{
                "train": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/train.zip",
                "validation": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/valid.zip",
            }}"""
    )

    mini_urls = (
        f"""{{
                "train": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/valid-mini.zip",
                "validation": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/valid-mini.zip",
                "test": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/valid-mini.zip",
            }}"""
        if "test" in roboflow_dataset.splits.keys()
        else f"""{{
                "train": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/valid-mini.zip",
                "validation": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/valid-mini.zip",
            }}"""
    )

    splits = (
        """[
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "folder_dir": data_files["train"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "folder_dir": data_files["validation"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "folder_dir": data_files["test"],
                },
            ),
]"""
        if "test" in roboflow_dataset.splits.keys()
        else """[
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "folder_dir": data_files["train"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "folder_dir": data_files["validation"],
                },
            ),
]"""
    )

    detection_dataset_script_template = f'''import collections
import json
import os

import datasets


_HOMEPAGE = "{dataset_url}"
_LICENSE = "{license}"
_CITATION = """\\
{citation}
"""
_CATEGORIES = {dataset_labels}
_ANNOTATION_FILENAME = "_annotations.coco.json"


class {dataset_name.upper().replace('-', '')}Config(datasets.BuilderConfig):
    """Builder Config for {dataset_name}"""

    def __init__(self, data_urls, **kwargs):
        """
        BuilderConfig for {dataset_name}.

        Args:
          data_urls: `dict`, name to url to download the zip file from.
          **kwargs: keyword arguments forwarded to super.
        """
        super({dataset_name.upper().replace('-', '')}Config, self).__init__(version=datasets.Version("1.0.0"), **kwargs)
        self.data_urls = data_urls


class {dataset_name.upper().replace('-', '')}(datasets.GeneratorBasedBuilder):
    """{dataset_name} instance segmentation dataset"""

    VERSION = datasets.Version("1.0.0")
    BUILDER_CONFIGS = [
        {dataset_name.upper().replace('-', '')}Config(
            name="full",
            description="Full version of {dataset_name} dataset.",
            data_urls={urls},
        ),
        {dataset_name.upper().replace('-', '')}Config(
            name="mini",
            description="Mini version of {dataset_name} dataset.",
            data_urls={mini_urls},
        )
    ]

    def _info(self):
        features = datasets.Features(
            {{
                "image_id": datasets.Value("int64"),
                "image": datasets.Image(),
                "width": datasets.Value("int32"),
                "height": datasets.Value("int32"),
                "objects": datasets.Sequence(
                    {{
                        "id": datasets.Value("int64"),
                        "area": datasets.Value("int64"),
                        "bbox": datasets.Sequence(datasets.Value("float32"), length=4),
                        "segmentation": datasets.Sequence(datasets.Sequence(datasets.Value("float32"))),
                        "category": datasets.ClassLabel(names=_CATEGORIES),
                    }}
                ),
            }}
        )
        return datasets.DatasetInfo(
            features=features,
            homepage=_HOMEPAGE,
            citation=_CITATION,
            license=_LICENSE,
        )

    def _split_generators(self, dl_manager):
        data_files = dl_manager.download_and_extract(self.config.data_urls)
        return {splits}

    def _generate_examples(self, folder_dir):
        def process_annot(annot, category_id_to_category):
            return {{
                "id": annot["id"],
                "area": annot["area"],
                "bbox": annot["bbox"],
                "segmentation": annot["segmentation"],
                "category": category_id_to_category[annot["category_id"]],
            }}

        image_id_to_image = {{}}
        idx = 0

        annotation_filepath = os.path.join(folder_dir, _ANNOTATION_FILENAME)
        with open(annotation_filepath, "r") as f:
            annotations = json.load(f)
        category_id_to_category = {{category["id"]: category["name"] for category in annotations["categories"]}}
        image_id_to_annotations = collections.defaultdict(list)
        for annot in annotations["annotations"]:
            image_id_to_annotations[annot["image_id"]].append(annot)
        filename_to_image = {{image["file_name"]: image for image in annotations["images"]}}

        for filename in os.listdir(folder_dir):
            filepath = os.path.join(folder_dir, filename)
            if filename in filename_to_image:
                image = filename_to_image[filename]
                objects = [
                    process_annot(annot, category_id_to_category) for annot in image_id_to_annotations[image["id"]]
                ]
                with open(filepath, "rb") as f:
                    image_bytes = f.read()
                yield idx, {{
                    "image_id": image["id"],
                    "image": {{"path": filepath, "bytes": image_bytes}},
                    "width": image["width"],
                    "height": image["height"],
                    "objects": objects,
                }}
                idx += 1
'''

    with open(Path(export_dir) / f"{dataset_name}.py", "w") as f:
        f.write(detection_dataset_script_template)



from roboflow2huggingface.hf_utils.instance_segmentation_dataset import (
    export_hf_instance_segmentation_dataset_script,
)
from roboflow2huggingface.hf_utils.object_detection_dataset import (
    export_hf_object_detection_dataset_script,
)
from roboflow2huggingface.hf_utils.image_classification_dataset import (
    export_hf_image_classification_dataset_script,
)


def export_hf_dataset_script(
    hf_dataset_id,
    dataset_labels,
    roboflow_dataset,
    export_dir,
    roboflow_universe_url,
    task="object-detection",
):
    if task == "object-detection":
        export_hf_object_detection_dataset_script(
            hf_dataset_id=hf_dataset_id,
            dataset_labels=dataset_labels,
            roboflow_dataset=roboflow_dataset,
            export_dir=export_dir,
            roboflow_universe_url=roboflow_universe_url,
        )
    elif task == "instance-segmentation":
        export_hf_instance_segmentation_dataset_script(
            hf_dataset_id=hf_dataset_id,
            dataset_labels=dataset_labels,
            roboflow_dataset=roboflow_dataset,
            export_dir=export_dir,
            roboflow_universe_url=roboflow_universe_url,
        )
    elif task == "image-classification":
        export_hf_image_classification_dataset_script(
            hf_dataset_id=hf_dataset_id,
            dataset_labels=dataset_labels,
            roboflow_dataset=roboflow_dataset,
            export_dir=export_dir,
            roboflow_universe_url=roboflow_universe_url,
        )
    else:
        raise ValueError(f"Unknown task: {task}")



import logging
import os
import random
import shutil
import json
from typing import List
import requests
import urllib
from bs4 import BeautifulSoup
import urllib
import roboflow

from pathlib import Path

ROBOFLOW_DATASETTXT_FILENAME = "README.dataset.txt"
ROBOFLOW_ROBOFLOWTXT_FILENAME = "README.roboflow.txt"
ROBOFLOW_UNIVERSE_BASE_URL = "https://universe.roboflow.com"
ROBOFLOW_ROBOFLOWTXT_CONTENT_START = "This dataset was exported via roboflow"

LOGGER = logging.getLogger(__name__)


def parse_citation(universe_url) -> str:
    """
    Parses the citation from a Roboflow Universe URL.

    Args:
        universe_url (str): Roboflow Universe URL
            example: https://universe.roboflow.com/boxer5/123-qq5ea/dataset/6
            example: https://universe.roboflow.com/boxer5/123-qq5ea/

    Returns:
        citation (str): Citation for the dataset
    """

    url = urllib.parse.urlparse(universe_url)
    base_url = url[0] + "://" + url[1]
    if base_url != ROBOFLOW_UNIVERSE_BASE_URL:
        raise ValueError(f"Not a valid Roboflow Universe URL: {universe_url}")

    if "/dataset/" in universe_url:
        project_url = universe_url.split("/dataset")[0] + "/"
    else:
        project_url = universe_url
    request = requests.get(project_url)
    soup = BeautifulSoup(request.text, "html.parser")
    possible_citation_components = soup.find_all("code")
    citation_text = ""
    for possible_citation_component in possible_citation_components:
        possible_citation_text = possible_citation_component.get_text(strip=True)
        if possible_citation_text.startswith("@misc{"):
            citation_text = possible_citation_text.replace("\\", "\\\\")
            break
    return citation_text


def parse_tags(universe_url) -> List[str]:
    """
    Parses the tags from a Roboflow Universe URL.

    Args:
        universe_url (str): Roboflow Universe URL
            example: https://universe.roboflow.com/boxer5/123-qq5ea/dataset/6
            example: https://universe.roboflow.com/boxer5/123-qq5ea/

    Returns:
        tags (List[str]): Tags for the dataset
    """

    url = urllib.parse.urlparse(universe_url)
    base_url = url[0] + "://" + url[1]
    if base_url != ROBOFLOW_UNIVERSE_BASE_URL:
        raise ValueError(f"Not a valid Roboflow Universe URL: {universe_url}")

    if "/dataset/" in universe_url:
        project_url = universe_url.split("/dataset")[0] + "/"
    else:
        project_url = universe_url
    request = requests.get(project_url)
    soup = BeautifulSoup(request.text, "html.parser")
    possible_tags_components = soup.find_all("a", class_="tag")
    tags = []
    for possible_tags_component in possible_tags_components:
        possible_tags_text = possible_tags_component.get_text(strip=True)
        if possible_tags_text != "Featured":
            tags.append(possible_tags_text)
    return tags


def download_roboflow_dataset(
    universe_url, api_key, location="roboflow_dataset"
) -> tuple:
    """
    Downloads a Roboflow dataset to a local directory.

    Args:
        universe_url (str): Roboflow Universe URL
            example: https://universe.roboflow.com/boxer5/123-qq5ea/dataset/6
        api_key (str): Roboflow API key
        location (str): Directory to download the dataset to

    Returns:
        project (roboflow.core.project.Project): Roboflow project object
        dataset (roboflow.core.version.Version): Roboflow dataset object
        task (str): Huggingface dataset task type. Supported types:
            ('image-classification', 'object-detection', 'instance-segmentation')
    """

    url = urllib.parse.urlparse(universe_url)
    base_url = url[0] + "://" + url[1]
    if base_url != ROBOFLOW_UNIVERSE_BASE_URL:
        raise ValueError(f"Not a valid Roboflow Universe URL: {universe_url}")
    _, workspace, project, _, version = url[2].split("/")

    rf = roboflow.Roboflow(api_key=api_key)
    project = rf.workspace(workspace).project(project)
    dataset = project.version(version)
    dataset_type = dataset.type
    if dataset_type == "classification":
        task = "image-classification"
        dataset.download("folder", location=location)
    elif dataset_type == "object-detection":
        task = "object-detection"
        dataset.download("coco", location=location)
    elif dataset_type == "instance-segmentation":
        task = "instance-segmentation"
        dataset.download("coco", location=location)
    else:
        raise ValueError(
            "Roboflow dataset type not supported {dataset_type}. Supported types: ('classification', 'object-detection', 'instance-segmentation')"
        )

    return project, dataset, task


def zip_roboflow_dataset(roboflow_dir, roboflow_dataset):
    """
    Zips the Roboflow dataset splits.

    Args:
        roboflow_dir (str): Path to the Roboflow dataset directory.
        roboflow_dataset (roboflow.core.version.Version): The Roboflow dataset object.
    """
    from pathlib import Path

    LOGGER.info("Zipping Roboflow dataset splits...")

    split_names = list(roboflow_dataset.splits.keys()) + ["valid-mini"]

    for split in split_names:
        source = Path(roboflow_dir) / split
        shutil.make_archive(
            Path(roboflow_dir) / "data" / split, format="zip", root_dir=source
        )
        shutil.rmtree(source)

    LOGGER.info("Roboflow dataset splits zipped!")


def read_roboflow_info(local_data_dir: str, roboflow_universe_url: str = None):
    """
    Reads the Roboflow dataset info from the README.dataset.txt and README.roboflow.txt files.

    Args:
        local_data_dir (str): Path to the Roboflow dataset directory.
        roboflow_universe_url (str): Roboflow Universe URL of the dataset.

    Returns:
        license (str): License of the dataset.
        dataset_url (str): Roboflow Universe URL of the dataset.
        citation (str): Citation of the dataset.
        roboflow_dataset_summary (str): Summary of the dataset.
        tags (List[str]): Tags of the dataset.
    """
    dataset_txt_path = Path(local_data_dir) / ROBOFLOW_DATASETTXT_FILENAME

    # read value of License: from README.dataset.txt
    with open(dataset_txt_path, "r") as f:
        lines = f.readlines()
        for line in lines:
            if line.startswith("License:"):
                license = line.split(":")[1].strip()
                break

    if roboflow_universe_url is None:
        # read dataset url from README.dataset.txt
        with open(dataset_txt_path, "r") as f:
            lines = f.readlines()
            for line in lines:
                if line.startswith(ROBOFLOW_UNIVERSE_BASE_URL):
                    dataset_url = line.strip()
                    break
    else:
        dataset_url = roboflow_universe_url

    # parse citation info from dataset_url
    citation = parse_citation(dataset_url)

    # parse tags from dataset_url
    tags = parse_tags(dataset_url)

    roboflow_txt_path = Path(local_data_dir) / ROBOFLOW_ROBOFLOWTXT_FILENAME

    # combine all lines starting from expression expression This dataset was exported via roboflow.com from README.roboflow.txt
    with open(roboflow_txt_path, "r") as f:
        lines = f.readlines()
        for i, line in enumerate(lines):
            if line.startswith(ROBOFLOW_ROBOFLOWTXT_CONTENT_START):
                start = i
                break
        roboflow_dataset_summary = "".join(lines[start:])

    return license, dataset_url, citation, roboflow_dataset_summary, tags


def fix_annotation_file(source_path: str):
    """
    Fixes the coco annotation json file by removing first category if its supercategory is 'none'.
    Also updates remaining category ids as 0 to N.
    Then, updates all category ids in annotations.
    Lastly, removes images and annotations that contain null bbox.

    Args:
        source_path (str): Path to the annotation file.
    """
    with open(source_path, "r") as f:
        data = json.load(f)

    # remove first category if its supercategory is 'none'
    if data["categories"][0]["supercategory"] == "none":
        data["categories"].pop(0)

        # update category ids
        for i, category in enumerate(data["categories"]):
            category["id"] = i

        # update annotation category ids
        for annotation in data["annotations"]:
            annotation["category_id"] = annotation["category_id"] - 1

    images = data["images"]
    annotations = data["annotations"]

    # remove images and annotations that contain null bbox
    new_images = []
    new_annotations = []

    invalid_image_ids = set()
    for annotation in annotations:
        if len(annotation["bbox"]) != 4:
            invalid_image_ids.add(annotation["image_id"])
        else:
            new_annotations.append(annotation)

    for image in images:
        if image["id"] not in invalid_image_ids:
            new_images.append(image)

    data["images"] = new_images
    data["annotations"] = new_annotations

    with open(source_path, "w") as f:
        json.dump(data, f)


def fix_annotations_files(local_data_dir: str):
    """
    Fixes the coco annotation json files by removing first category if its supercategory is 'none'.
    Also updates remaining category ids as 0 to N.
    Lastly, updates all category ids in annotations.

    Args:
        local_data_dir (str): Path to the Roboflow dataset directory.
    """

    for split in ["train", "valid", "test"]:
        source_path = Path(local_data_dir) / split / "_annotations.coco.json"
        if source_path.exists():
            fix_annotation_file(source_path)


def normalize_dataset_labels(local_data_dir: str) -> None:
    """
    Normalizes the dataset labels by converting them to lowercase.

    Args:
        local_data_dir (str): Path to the Roboflow dataset directory.
    """
    for split in ["train", "valid", "test"]:
        source_path = Path(local_data_dir) / split / "_annotations.coco.json"
        if source_path.exists():
            with open(source_path, "r") as f:
                data = json.load(f)
                for coco_category in data["categories"]:
                    coco_category["name"] = coco_category["name"].lower()

            with open(source_path, "w") as f:
                json.dump(data, f)


def read_dataset_labels_from_coco(local_data_dir: str) -> List[str]:
    """
    Reads the dataset labels from the coco annotation json file.

    Args:
        local_data_dir (str): Path to the Roboflow dataset directory.

    Returns:
        dataset_labels (List[str]): List of dataset labels.
    """
    train_coco_json_path = Path(local_data_dir) / "train" / "_annotations.coco.json"
    with open(train_coco_json_path, "r") as f:
        data = json.load(f)
    dataset_labels = [coco_category["name"] for coco_category in data["categories"]]

    return dataset_labels


def create_mini_valid_split_from_coco(local_data_dir, num_samples: int = 3):
    """
    Creates a mini valid split in coco format by copying random samples from valid split.

    Args:
        local_data_dir (str): Path to the Roboflow dataset directory.
        num_samples (int): Number of samples to copy from valid split.
    """

    valid_coco_json_path = Path(local_data_dir) / "valid" / "_annotations.coco.json"
    with open(valid_coco_json_path, "r") as f:
        data = json.load(f)

    images = data["images"]
    annotations = data["annotations"]

    # get random samples from valid split
    random.shuffle(images)

    mini_valid_images = images[:num_samples]

    # gen mini valid annotations
    mini_valid_annotations = []
    for annotation in annotations:
        if annotation["image_id"] in [image["id"] for image in mini_valid_images]:
            mini_valid_annotations.append(annotation)

    # create mini valid split
    mini_valid_split_dir = Path(local_data_dir) / "valid-mini"
    mini_valid_split_dir.mkdir(exist_ok=True)

    # copy images
    for image in mini_valid_images:
        image_path = Path(local_data_dir) / "valid" / image["file_name"]
        shutil.copy(image_path, mini_valid_split_dir)

    # copy annotations
    data["images"] = mini_valid_images
    data["annotations"] = mini_valid_annotations

    with open(mini_valid_split_dir / "_annotations.coco.json", "w") as f:
        json.dump(data, f)


def create_mini_valid_split_from_folder(local_data_dir, num_samples_per_label: int = 2):
    """
    Creates a mini valid split in folder format by copying random samples from valid split.

    Args:
        local_data_dir (str): Path to the Roboflow dataset directory.
        num_samples_per_label (int): Number of samples to copy from valid split per label.
    """

    valid_split_dir = Path(local_data_dir) / "valid"
    mini_valid_split_dir = Path(local_data_dir) / "valid-mini"
    mini_valid_split_dir.mkdir(exist_ok=True)

    # copy images (can be jpg png jpeg etc.)
    labels = os.listdir(valid_split_dir)
    for label in labels:
        label_dir = valid_split_dir / label
        images = os.listdir(label_dir)
        random.shuffle(images)
        mini_valid_images = images[:num_samples_per_label]
        target_base_image_name = "image.jpg"
        for ind, image in enumerate(mini_valid_images):
            source_image_path = label_dir / image
            target_image_path = (
                mini_valid_split_dir / label / f"{target_base_image_name}{ind}.jpg"
            )
            target_image_path.parent.mkdir(exist_ok=True)
            shutil.copy(source_image_path, target_image_path)



from pathlib import Path
import json
from typing import List
from roboflow2huggingface.roboflow_utils import read_roboflow_info


def export_hf_dataset_card(
    dataset_labels: List[str],
    export_dir: str,
    hf_dataset_id: str,
    task="object-detection",
    roboflow_universe_url: str = None,
    split_name_to_num_samples: dict = None,
):
    """
    Exports a dataset card to the specified directory.

    Args:
        dataset_labels List[str]: The labels of the dataset.
        export_dir (str): Path to the directory to export the dataset card to.
        hf_dataset_id (str,): The Hugging Face dataset ID.
        task (str, optional): The task of the dataset. Defaults to "object-detection".
        roboflow_universe_url (str, optional): The Roboflow Universe URL. Defaults to None.
        split_name_to_num_samples (dict, optional): A dictionary mapping split names to the number of samples in the split. Defaults to None.
    """
    Path(export_dir).mkdir(parents=True, exist_ok=True)

    hf_task = "image-segmentation" if task == "instance-segmentation" else task

    # export split_name_to_num_samples as json
    if split_name_to_num_samples is not None:
        with open(f"{export_dir}/split_name_to_num_samples.json", "w") as f:
            json.dump(split_name_to_num_samples, f)

    # parse reboflow info
    (
        license,
        dataset_url,
        citation,
        roboflow_dataset_summary,
        roboflow_tags,
    ) = read_roboflow_info(
        local_data_dir=export_dir, roboflow_universe_url=roboflow_universe_url
    )

    # generate tags string line by line
    roboflow_tags_str = ""
    for ind, roboflow_tag in enumerate(roboflow_tags):
        roboflow_tags_str += f"- {roboflow_tag}"
        if ind != len(roboflow_tags) - 1:
            roboflow_tags_str += "\n"

    if split_name_to_num_samples is not None:
        num_samples_line = f"""
### Number of Images

```json
{split_name_to_num_samples}
```
"""
    else:
        num_samples_line = ""

    card = f"""---
task_categories:
- {hf_task}
tags:
- roboflow
- roboflow2huggingface
{roboflow_tags_str}
---

<div align="center">
  <img width="640" alt="{hf_dataset_id}" src="https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/thumbnail.jpg">
</div>

### Dataset Labels

```
{dataset_labels}
```

{num_samples_line}

### How to Use

- Install [datasets](https://pypi.org/project/datasets/):

```bash
pip install datasets
```

- Load the dataset:

```python
from datasets import load_dataset

ds = load_dataset("{hf_dataset_id}", name="full")
example = ds['train'][0]
```

### Roboflow Dataset Page
[{dataset_url}]({dataset_url}?ref=roboflow2huggingface)

### Citation

```
{citation}
```

### License
{license}

### Dataset Summary
{roboflow_dataset_summary}
"""

    with open(Path(export_dir) / "README.md", "w") as f:
        f.write(card)



from pathlib import Path

FONT_PATH = Path(__file__).parent / "OpenSans-Bold.ttf"


def extract_random_test_images_from_zip(
    local_data_dir, export_dir, num_samples: int = 3
):
    import random
    import zipfile

    zip_filepath = Path(local_data_dir) / "data" / "valid.zip"

    with zipfile.ZipFile(zip_filepath) as zf:
        image_files = zf.namelist()
        image_files.remove("_annotations.coco.json")
        selected_image_files = random.sample(image_files, num_samples)
        for image_file in selected_image_files:
            zf.extract(image_file, path=export_dir)

    return export_dir


def extract_random_image_from_valid_zip(
    local_data_dir, export_dir, task="object-detection"
):
    import random
    import zipfile

    zip_filepath = Path(local_data_dir) / "data" / "valid.zip"

    if task in ["object-detection", "instance-segmentation"]:
        with zipfile.ZipFile(zip_filepath) as zf:
            image_files = zf.namelist()
            image_files.remove("_annotations.coco.json")
            selected_image_file = random.sample(image_files, 1)[0]
            image_path = zf.extract(selected_image_file, path=export_dir)

        return image_path
    elif task == "image-classification":
        with zipfile.ZipFile(zip_filepath) as zf:
            image_files = zf.namelist()
            selected_image_file = random.sample(image_files, 1)[0]
            image_path = zf.extract(selected_image_file, path=export_dir)

        return image_path


def add_text_to_image(
    text,
    pil_image,
    brightness: float = 0.75,
    text_font: int = 50,
    crop_margin: int = None,
):
    from PIL import ImageDraw, ImageEnhance, ImageFont

    RESIZE_WIDTH_TO = 1280
    MAX_TEXT_WIDTH = 860

    # Create an ImageEnhance object
    enhancer = ImageEnhance.Brightness(pil_image)

    # Darken the image by 25%
    pil_image = enhancer.enhance(brightness)

    # Resize the image
    pil_image = pil_image.resize((RESIZE_WIDTH_TO, RESIZE_WIDTH_TO))

    # crop image
    if crop_margin is not None:
        width, height = pil_image.size
        mid_height = int(height / 2)
        min_height = max(0, mid_height - crop_margin)
        max_height = min(height, mid_height + crop_margin)
        pil_image = pil_image.crop((0, min_height, width, max_height))

    # Create an ImageDraw object
    draw = ImageDraw.Draw(pil_image)

    # Define the font and text to be written
    font = ImageFont.truetype(str(FONT_PATH), text_font)

    # Get the bounding box of the text
    bbox = draw.textbbox((0, 0), text, font=font)

    # Get the width and height of the text
    text_width = bbox[2] - bbox[0]
    text_height = bbox[3] - bbox[1]

    # update font size
    if text_width > MAX_TEXT_WIDTH:
        text_font = int(MAX_TEXT_WIDTH / text_width * text_font)
        font = ImageFont.truetype(str(FONT_PATH), text_font)
        bbox = draw.textbbox((0, 0), text, font=font)
        text_width = bbox[2] - bbox[0]
        text_height = bbox[3] - bbox[1]

    # Define the coordinates of the smaller rounded rectangle
    box_margin = 20
    x1, y1 = (pil_image.width - text_width) / 2 - box_margin, (
        pil_image.height - text_height
    ) / 2 - box_margin / 3
    x2, y2 = (pil_image.width + text_width) / 2 + box_margin, (
        pil_image.height + text_height * 2
    ) / 2 + box_margin / 3

    # Define the radius of the rounded corners
    radius = 15

    # Draw the rounded rectangle
    draw.rounded_rectangle(
        [(x1, y1), (x2, y2)],
        radius=radius,
        fill=(255, 255, 255),
        outline=None,
        width=5,
    )

    # Draw the text on the image
    draw.text(
        ((pil_image.width - text_width) / 2, (pil_image.height - text_height) / 2),
        text,
        font=font,
        fill=(0, 0, 0),
    )

    return pil_image



import logging
import os
import shutil

from roboflow2huggingface.hf_utils.dataset_script import \
    export_hf_dataset_script

from .hf_utils.dataset_card import export_hf_dataset_card
from .hf_utils.hub import export_thumbnail, upload_dataset_to_hfhub
from .other_utils import extract_random_image_from_valid_zip
from .roboflow_utils import (create_mini_valid_split_from_coco,
                             create_mini_valid_split_from_folder,
                             download_roboflow_dataset, fix_annotations_files,
                             normalize_dataset_labels,
                             read_dataset_labels_from_coco,
                             zip_roboflow_dataset)
from .yolov5_utils import make_dataset_yolov5_trainable

LOGLEVEL = os.environ.get("LOGLEVEL", "INFO").upper()
logging.basicConfig(level=LOGLEVEL)
LOGGER = logging.getLogger(__name__)


__version__ = "0.0.21"


def roboflow_to_huggingface_pipeline(
    roboflow_universe_url: str,
    hf_dataset_id: str,
    roboflow_api_key: str = None,
    local_data_dir: str = "roboflow_dataset",
    hf_private: bool = False,
    hf_write_token: str = None,
    keep_local: bool = False,
    make_yolov5_trainable: bool = False,
    mini_valid_split_sample_num: int = 3,
):
    """
    Downloads a Roboflow dataset and uploads it to the Hugging Face Hub.

    Args:
        roboflow_universe_url (str): The Roboflow universe URL.
        hf_dataset_id (str): The name of the dataset on the Hugging Face Hub.
        roboflow_api_key (str, optional): The Roboflow API key. Defaults to None.
        local_data_dir (str, optional): The local directory to download the dataset to. Defaults to "roboflow_dataset".
        hf_private (bool, optional): Whether the dataset should be private on the Hugging Face Hub. Defaults to False.
        hf_write_token (str, optional): The token to use to authenticate to the Hugging Face Hub. Defaults to None.
        keep_local (bool, optional): Whether to keep the local dataset. Defaults to False.
        make_yolov5_trainable (bool, optional): Whether to convert the dataset to a YOLOv5 trainable format. Defaults to False.
        mini_valid_split_sample_num (int, optional): The number of samples to use for the mini validation split. Defaults to 3.
    """
    roboflow_api_key = roboflow_api_key or os.environ.get("ROBOFLOW_API_KEY")

    roboflow_project, roboflow_dataset, task = download_roboflow_dataset(
        roboflow_universe_url, api_key=roboflow_api_key, location=local_data_dir
    )
    normalize_dataset_labels(local_data_dir)
    fix_annotations_files(local_data_dir)

    # create mini validation split
    if task in ["object-detection", "instance-segmentation"]:
        create_mini_valid_split_from_coco(
            local_data_dir, num_samples=mini_valid_split_sample_num
        )
    elif task == "image-classification":
        create_mini_valid_split_from_folder(local_data_dir)

    # read dataset labels
    if task in ["object-detection", "instance-segmentation"]:
        dataset_labels = read_dataset_labels_from_coco(local_data_dir)
    elif task == "image-classification":
        dataset_labels = list(roboflow_project.classes.keys())

    zip_roboflow_dataset(local_data_dir, roboflow_dataset=roboflow_dataset)

    # export dataset python script
    export_hf_dataset_script(
        hf_dataset_id=hf_dataset_id,
        dataset_labels=dataset_labels,
        roboflow_dataset=roboflow_dataset,
        export_dir=local_data_dir,
        roboflow_universe_url=roboflow_universe_url,
        task=task,
    )

    # export thumbnail
    image_path = extract_random_image_from_valid_zip(
        local_data_dir, export_dir=local_data_dir, task=task
    )
    export_thumbnail(image_path=image_path, repo_id=hf_dataset_id, task=task)
    os.remove(image_path)

    export_hf_dataset_card(
        dataset_labels=dataset_labels,
        export_dir=local_data_dir,
        task=task,
        roboflow_universe_url=roboflow_universe_url,
        hf_dataset_id=hf_dataset_id,
        split_name_to_num_samples=roboflow_dataset.splits,
    )

    upload_dataset_to_hfhub(
        dataset_dir=local_data_dir,
        repo_id=hf_dataset_id,
        token=hf_write_token,
        private=hf_private,
    )

    if not keep_local:
        shutil.rmtree(local_data_dir)

    if make_yolov5_trainable and keep_local:
        LOGGER.info("Making dataset YOLOv5 trainable...")
        datayaml_path = make_dataset_yolov5_trainable(
            roboflow_dataset_dir=local_data_dir
        )

        LOGGER.info(
            f"YOLOv5 training data is ready! You can find the data.yaml file at {datayaml_path}"
        )

        return datayaml_path



import fire
from roboflow2huggingface import roboflow_to_huggingface_pipeline


def app() -> None:
    """Cli app."""
    fire.Fire(roboflow_to_huggingface_pipeline)


if __name__ == "__main__":
    app()



from roboflow2huggingface.roboflow_utils import read_roboflow_info
from pathlib import Path


def export_hf_image_classification_dataset_script(
    hf_dataset_id,
    dataset_labels,
    roboflow_dataset,
    export_dir,
    roboflow_universe_url: str = None,
):
    """
    Exports a HuggingFace dataset script for a Roboflow image classification dataset.

    Args:
        hf_dataset_id (str): HuggingFace dataset id
        dataset_labels (list): List of labels as strings
        roboflow_dataset (roboflow.core.version.Version): Roboflow dataset object
        export_dir (str): Directory to export the dataset script to
        roboflow_universe_url (str): Roboflow Universe URL
    """

    license, dataset_url, citation, _, _ = read_roboflow_info(
        local_data_dir=export_dir, roboflow_universe_url=roboflow_universe_url
    )
    dataset_name = hf_dataset_id.split("/")[-1]
    urls = (
        f"""{{
    "train": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/train.zip",
    "validation": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/valid.zip",
    "test": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/test.zip",
}}
"""
        if "test" in roboflow_dataset.splits.keys()
        else f"""{{
    "train": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/train.zip",
    "validation": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/valid.zip",
}}
"""
    )

    mini_urls = (
        f"""{{
                "train": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/valid-mini.zip",
                "validation": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/valid-mini.zip",
                "test": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/valid-mini.zip",
            }}"""
        if "test" in roboflow_dataset.splits.keys()
        else f"""{{
                "train": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/valid-mini.zip",
                "validation": "https://huggingface.co/datasets/{hf_dataset_id}/resolve/main/data/valid-mini.zip",
            }}"""
    )

    splits = (
        """[
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "files": dl_manager.iter_files([data_files["train"]]),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "files": dl_manager.iter_files([data_files["validation"]]),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "files": dl_manager.iter_files([data_files["test"]]),
                },
            ),
]"""
        if "test" in roboflow_dataset.splits.keys()
        else """[
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "files": dl_manager.iter_files([data_files["train"]]),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "files": dl_manager.iter_files([data_files["validation"]]),
                },
            ),
]"""
    )

    classification_dataset_script_template = f'''import os

import datasets
from datasets.tasks import ImageClassification


_HOMEPAGE = "{dataset_url}"
_LICENSE = "{license}"
_CITATION = """\\
{citation}
"""
_CATEGORIES = {dataset_labels}


class {dataset_name.upper().replace('-', '')}Config(datasets.BuilderConfig):
    """Builder Config for {dataset_name}"""

    def __init__(self, data_urls, **kwargs):
        """
        BuilderConfig for {dataset_name}.

        Args:
          data_urls: `dict`, name to url to download the zip file from.
          **kwargs: keyword arguments forwarded to super.
        """
        super({dataset_name.upper().replace('-', '')}Config, self).__init__(version=datasets.Version("1.0.0"), **kwargs)
        self.data_urls = data_urls


class {dataset_name.upper().replace('-', '')}(datasets.GeneratorBasedBuilder):
    """{dataset_name} image classification dataset"""

    VERSION = datasets.Version("1.0.0")
    BUILDER_CONFIGS = [
        {dataset_name.upper().replace('-', '')}Config(
            name="full",
            description="Full version of {dataset_name} dataset.",
            data_urls={urls},
        ),
        {dataset_name.upper().replace('-', '')}Config(
            name="mini",
            description="Mini version of {dataset_name} dataset.",
            data_urls={mini_urls},
        )
    ]

    def _info(self):
        return datasets.DatasetInfo(
            features=datasets.Features(
                {{
                    "image_file_path": datasets.Value("string"),
                    "image": datasets.Image(),
                    "labels": datasets.features.ClassLabel(names=_CATEGORIES),
                }}
            ),
            supervised_keys=("image", "labels"),
            homepage=_HOMEPAGE,
            citation=_CITATION,
            license=_LICENSE,
            task_templates=[ImageClassification(image_column="image", label_column="labels")],
        )

    def _split_generators(self, dl_manager):
        data_files = dl_manager.download_and_extract(self.config.data_urls)
        return {splits}

    def _generate_examples(self, files):
        for i, path in enumerate(files):
            file_name = os.path.basename(path)
            if file_name.endswith((".jpg", ".png", ".jpeg", ".bmp", ".tif", ".tiff")):
                yield i, {{
                    "image_file_path": path,
                    "image": path,
                    "labels": os.path.basename(os.path.dirname(path)),
                }}
'''

    with open(Path(export_dir) / f"{dataset_name}.py", "w") as f:
        f.write(classification_dataset_script_template)
